#include "bootloader.h"

//bootloader code used from micropython machine_bootloader

// Location in RAM of bootloader state (just after the top of the stack).
// STM32H7 has ECC and writes to RAM must be 64-bit so they are fully committed
// to actual SRAM before a system reset occurs.
#define BL_STATE_PTR                ((uint64_t *)&_estack)
#define BL_STATE_KEY                (0x5a5) //arbitrary bit pattern used as marker
#define BL_STATE_KEY_MASK           (0xfff)
#define BL_STATE_KEY_SHIFT          (32)
#define BL_STATE_INVALID            (0)
#define BL_STATE_VALID(reg, addr)   ((uint64_t)(reg) | ((uint64_t)((addr) | BL_STATE_KEY)) << BL_STATE_KEY_SHIFT)
#define BL_STATE_GET_REG(s)         ((s) & 0xffffffff)
#define BL_STATE_GET_KEY(s)         (((s) >> BL_STATE_KEY_SHIFT) & BL_STATE_KEY_MASK)
#define BL_STATE_GET_ADDR(s)        (((s) >> BL_STATE_KEY_SHIFT) & ~BL_STATE_KEY_MASK)
extern uint64_t _estack[];


void branch_to_bootloader(uint32_t r0, uint32_t bl_addr){
    __asm volatile (
        "ldr r2, [r1, #0]\n"    // get address of stack pointer
        "msr msp, r2\n"         // get stack pointer
        "ldr r2, [r1, #4]\n"    // get address of destination
        "bx r2\n"               // branch to bootloader
        );
    //unreachable code
    while(1);
}

//check if memory is set after software reboot indicating we need to branch to the bootloader
void check_enter_bootloader(){
    uint64_t bl_state = *BL_STATE_PTR;
    //set to invalid for next boot
    *BL_STATE_PTR = BL_STATE_INVALID;

    //TODO test if statement
    if(BL_STATE_GET_KEY(bl_state) == BL_STATE_KEY && (RCC->CSR & RCC_CSR_SFTRSTF)){
        //botloader data valid and was just reset with NVIC_SystemReset

        //remap memory to system flash
        SYSCFG_MemoryRemapConfig(SYSCFG_MemoryRemap_SystemFlash);
        branch_to_bootloader(BL_STATE_GET_REG(bl_state), BL_STATE_GET_ADDR(bl_state));
    }
}

void enter_bootloader(uint32_t r0, uint32_t bl_addr){
    //set bootloader state values    
    *BL_STATE_PTR = BL_STATE_VALID(r0, bl_addr);

    NVIC_SystemReset();
}